﻿

##############################################################################
# PowerShell の StrictMode に設定
##############################################################################
Set-StrictMode -Version Latest

##############################################################################
# デフォルトの設定値の定義
##############################################################################
$conf = @{
	"username" = "";
	"domainFilter" = "";
	"reuseCred" = "1";
	"logFileName" = "ps-%Y%m%d.log";
    "logTimeDisplay" = "%Y/%m/%d %H:%M:%S:%M";
    "logSavingDays" = "7";
    "timeOutValue" = "3600";

}

##############################################################################
# ファイル名の定義
##############################################################################
$configfile = "config.ini"
$tmpdir = "tmp"
$credfile = "tmp\cred.xml"
$logdir = "log"


##############################################################################
# 関数
##############################################################################

function log($str) {
    if (-not (test-path $logdir)) {
        # ログ用ディレクトリの作成
        New-Item $logdir -ItemType directory -Force
        log("Create log directory")
    }
    $logTimeDisplay = Get-Date -UFormat $conf.logTimeDisplay
    $str = $logTimeDisplay + " " + $str
	$Host.UI.WriteLine($str)
	$logname = Get-Date -UFormat $conf.logFileName
    $logfile = $logdir + "\" + $logname
	Write-Output $str | out-file $logfile Default -append
}

function debugPrint($str) {
	if ($conf.debug) {
		log($str)
	}
}

function deleteLog()
{
    $today = Get-Date
    if (Test-Path($logdir)) {
        $logs = Get-ChildItem $logdir
        foreach ($log in $logs) {
            if ($log.lastWriteTime -lt $today.AddDays(-$conf.logSavingDays)) {
                Remove-Item $log.FullName -Force
                log("Deleted log file=" + $log.Name)
            }
        }
    }
}





########################################
# 設定ファイルの読み込み
########################################
function readConfig() {
	if (-not (test-path $configfile)) {
		return;
	}
	
	log("Read configuration file：" + $configfile)
	$lines = Get-Content $configfile
	foreach($line in $lines){
		# コメントと空行を除外する
		if ($line -match "^$") { continue }
		if ($line -match "^\s*;") { continue }
		$param = $line.split("=", 2);
		$conf[$param[0].Trim()] = $param[1].Trim();
	}
}


########################################
# timestamp登録&確認処理
########################################

function RegistTimestamp() {

#timepstamp登録処理

#ドメイン未指定
if ($conf.domainFilter -eq ""){

	log("No domain setting" +$conf.domainFilter)
	$userList = Get-MsolUser -ALL | select-object UserPrincipalName,StsRefreshTokensValidFrom,LastPasswordChangeTimeStamp  | where-object {$_.lastpasswordchangetimestamp -eq $Null -or $_.StsRefreshTokensValidFrom -eq $Null}

#ドメイン指定
} else {

	log("domain setting:" +$conf.domainFilter)
	$domainPrm = "*@" + $conf.domainFilter

	$userList = Get-MsolUser -ALL | select-object UserPrincipalName,StsRefreshTokensValidFrom,LastPasswordChangeTimeStamp  | where-object {($_.UserPrincipalName -like $domainPrm) -and ($_.lastpasswordchangetimestamp -eq $Null -or $_.StsRefreshTokensValidFrom -eq $Null)}
}

#timestamp登録
$userList | foreach-object {
		log("Update target UPN=" + $_.userprincipalname + "  LastPasswordChangeTimeStamp=" + $_.lastpasswordchangetimestamp + "  StsRefreshTokensValidFrom=" + $_.StsRefreshTokensValidFrom)
		$mydate = Get-Date
		Set-MsolUser -UserPrincipalName $_.userprincipalname -StsRefreshTokensValidFrom $mydate -LastPasswordChangeTimeStamp $mydate 
	}

# timestamp登録確認処理

    $chkflg = $FALSE
	#開始時間
	[DateTime]$startTime = (Get-Date)
	log("timestamp check date=" + $startTime)
	
    while(-not $chkflg){
        
        $timeStampNonValue = $FALSE

        $userList | foreach-object {
            $ChkUserData =  Get-MsolUser -UserPrincipalName $_.userprincipalname | select-object UserPrincipalName,StsRefreshTokensValidFrom,LastPasswordChangeTimeStamp
             if (($ChkUserData.StsRefreshTokensValidFrom -eq $Null) -or ($ChkUserData.LastPasswordChangeTimeStamp -eq $Null)){
                $timeStampNonValue = $TRUE
            } else {
                log("Update complete UPN=" + $ChkUserData.UserPrincipalName + "  LastPasswordChangeTimeStamp=" + $ChkUserData.LastPasswordChangeTimeStamp + "  StsRefreshTokensValidFrom=" + $ChkUserData.StsRefreshTokensValidFrom)
            } 
        }

        if ($timeStampNonValue) {
            $timeStampNonValue = $FALSE
            
        } else {
            $chkflg = $TRUE
            log("Completed all users timestamp updated")
        }
        
        #タイムアウトの確認
        $prostime =  (New-TimeSpan $startTime (Get-Date)).TotalSeconds
        log("timestamp check loop process time=" + $prostime)
        if ($prostime -gt $conf.timeOutValue ) {
        	log("check loop process time out")
        	break
        }
    }
    

}


    


########################################
# Microsoft 365 に接続
########################################
function Connect365() {
	if ($conf.reuseCred -eq 0 -or -not (test-path $credfile)) {
		GET-CREDENTIAL -credential $conf.username | Export-Clixml $credfile
		if (-not $?) {
			log("CREDENTIAL file read error")
			exit 1
		}
	}
	$cred = Import-Clixml $credfile
	log("Log on user=" + $conf.username)
	Connect-MsolService -Credential $cred
	$ret = Get-MsolDomain -ErrorAction SilentlyContinue
	if (-not $?) {
		log("Connection failed")
		exit 1
	}
}


##############################################################################
# メイン
##############################################################################

# 設定ファイルの読み込み
log("Start timestamp tool execution")
if ($Args.length -gt 1 -and $Args[0] -eq "-f") {
  $configfile = $Args[1]
}
readConfig
#ログローテーション処理
deleteLog


if (-not (test-path $tmpdir)) {
    New-Item $tmpdir -ItemType directory -Force
    # 一時ディレクトリの作成
    log("Create a temporary directory")
}

# Microsoft365へ接続
Connect365

# timestampの登録
RegistTimestamp

log("Timestamp tool execution end")
